/******************************************************************************
 *
 * Copyright (C) 1996-2001 Jason Evans <jasone@canonware.com>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice(s), this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified other than the possible
 *    addition of one or more copyright notices.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice(s), this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 *
 * Version: 1.2.0
 *
 ******************************************************************************/

/* Pseudo-opaque type. */
typedef struct cw_thd_s cw_thd_t;

/*
 * Define whether to use the generic thread suspend/resume mechanism on
 * FreeBSD.
 */
#ifdef _CW_OS_FREEBSD
#define _CW_THD_FREEBSD_SR
#else
#define _CW_THD_GENERIC_SR
#endif

struct cw_thd_s {
#if (defined(_LIBSTASH_DBG) || defined(_LIBSTASH_DEBUG))
	cw_uint32_t	magic;
#endif
	pthread_t	thread;
	void		*(*start_func)(void *);
	void		*start_arg;
#ifdef _CW_THD_GENERIC_SR
	sem_t		sem;	/* For suspend/resume. */
#endif
	cw_bool_t	suspended;
	cw_mtx_t	crit_lock;
};

void	thd_new(cw_thd_t *a_thd, void *(*a_start_func)(void *), void *a_arg);
void	thd_delete(cw_thd_t *a_thd);
void	*thd_join(cw_thd_t *a_thd);
cw_thd_t *thd_self(void);
#define	thd_yield() sched_yield()
#define	thd_sigmask(a, b) pthread_sigmask((a), (b), NULL)
void	thd_crit_enter(void);
void	thd_crit_leave(void);
void	thd_suspend(cw_thd_t *a_thd);
cw_bool_t thd_trysuspend(cw_thd_t *a_thd);
void	thd_resume(cw_thd_t *a_thd);
