/******************************************************************************
 *
 * Copyright (C) 1996-2001 Jason Evans <jasone@canonware.com>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice(s), this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified other than the possible
 *    addition of one or more copyright notices.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice(s), this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 *
 * Version: 1.2.0
 *
 ******************************************************************************/

/* Pseudo-opaque type. */
typedef struct cw_treen_s cw_treen_t;

struct cw_treen_s {
#if (defined(_LIBSTASH_DBG) || defined(_LIBSTASH_DEBUG))
	cw_uint32_t	magic_a;
#endif

	/* Automatic deallocation hooks. */
	cw_opaque_dealloc_t *dealloc_func;
	void		*dealloc_arg;

	cw_treen_t	*parent;	/* Pointer to the parent. */

	/*
	 * Pointer to one child.  Getting to other children is achieved by
	 * iterating on the child's sibling qr.
	 */
	cw_treen_t	*child;

	qr(cw_treen_t) sib_link;	/* Linkage for the sibling ring. */

	void		*data;		/* The payload. */

#if (defined(_LIBSTASH_DBG) || defined(_LIBSTASH_DEBUG))
	cw_uint32_t	size_of;
	cw_uint32_t	magic_b;
#endif
};

cw_treen_t	*treen_new(cw_treen_t *a_treen, cw_mem_t *a_mem,
    cw_opaque_dealloc_t *a_dealloc_func, void *a_dealloc_arg);
void		treen_delete(cw_treen_t *a_treen);
void		treen_link(cw_treen_t *a_treen, cw_treen_t *a_parent);
cw_treen_t	*treen_parent_get(cw_treen_t *a_treen);
cw_treen_t	*treen_child_get(cw_treen_t *a_treen);
cw_treen_t	*treen_sibling_get(cw_treen_t *a_treen);
void		*treen_data_ptr_get(cw_treen_t *a_treen);
void		treen_data_ptr_set(cw_treen_t *a_treen, void *a_data);
