/******************************************************************************
 *
 * <Copyright = "jasone">
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice(s), this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified other than the possible
 *    addition of one or more copyright notices.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice(s), this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 *
 * Version: 1.2.0
 *
 ******************************************************************************/

/* Pseudo-opaque type. */
typedef struct cw_xep_s cw_xep_t;

typedef cw_sint32_t cw_xepv_t;

#define	_CW_XEPV_NONE		 0
#define	_CW_XEPV_CODE		-1
#define	_CW_XEPV_FINALLY	-2
#define	_CW_XEPV_OOM		-3
#define _CW_XEPV_OUT_PARSE	-4
/*
 * Exceptions should be numbered negatively, and care should be taken to avoid
 * duplicates.  -1 through -127 are reserved by libstash.
 */

typedef enum {
	_CW_XEPS_TRY,
	_CW_XEPS_CATCH,
	_CW_XEPS_FINALLY
} cw_xeps_t;

struct cw_xep_s {
	qr(cw_xep_t)	link;
	jmp_buf		context;
	cw_xepv_t	value;
	cw_bool_t	is_handled;
	cw_xeps_t	state;
	const char	*filename;
	cw_uint32_t	line_num;
};

#define xep_begin()							\
	{								\
		cw_xep_t	_xep

#define	xep_try								\
		xep_p_link(&_xep);					\
		switch (setjmp(_xep.context)) {				\
		case _CW_XEPV_NONE:					\
		case _CW_XEPV_CODE:

#define xep_catch(a_value)						\
			break;						\
		case (a_value):

#define xep_mcatch(a_value)						\
		case (a_value):

#define	xep_acatch							\
			break;						\
		default:

#define xep_finally xep_catch(_CW_XEPV_FINALLY)

#define	xep_end()							\
		}							\
		xep_p_unlink(&_xep);					\
	}

#define	xep_value()	(_xep.value)

void	xep_throw_e(cw_xepv_t a_value, const char *a_filename, cw_uint32_t
    a_line_num);
#define xep_throw(a_value)	xep_throw_e((a_value), __FILE__, __LINE__)
void	xep_retry(void);
void	xep_handled(void);

/*
 * Private, but visible here so that the cpp macros above don't cause
 * compilation warnings.
 */ 
void	xep_p_link(cw_xep_t *a_xep);
void	xep_p_unlink(cw_xep_t *a_xep);
